/* helpers.c
 * Copyright (C) 2008 Gauvain Pocentek <gpocentek@linutop.com>
 * This file is part of lbackup
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <mntent.h>
#include <regex.h>
#include <sys/types.h>
#include <dirent.h>

#include <glib.h>
#include <glib/gprintf.h>

#include <libhal-storage.h>

#define MTAB "/etc/mtab"

gint
compare (gconstpointer a, gconstpointer b)
{
    return strcmp ((char *)a, (char *)b);
}

int
get_root_device (char *root)
{
    struct mntent *fs;
    FILE *fp;
    int retval = 0;

    fp = setmntent (MTAB, "r");
    if (!fp)
        return 0;
    while ((fs = getmntent (fp)) != NULL) {
        if (strncmp (fs->mnt_dir, "/cdrom", 6) == 0) {
            strncpy (root, fs->mnt_fsname, 8);    /* "/dev/?d?" */
            root[8] = '\0';
            retval = 1;
            break;
        }
    }

    endmntent (fp);

    return retval;
}

GList *
get_valid_devices ()
{
    GList *list = NULL;
    regex_t RE;
    DIR *dev;
    struct dirent *dir;
    char fullpath[10];
    char root[9];

    if (!get_root_device (root))
        return NULL;

    regcomp (&RE, "^[sh]d[a-z]$", REG_EXTENDED);

    dev = opendir ("/dev");
    while ((dir = readdir (dev)) != NULL) {
        if (regexec (&RE, dir->d_name, 0, NULL, 0) == 0) {
            g_snprintf (fullpath, 10, "/dev/%s", dir->d_name);
            if (strcmp (fullpath, root) != 0) {    /* not the root FS */
                list = g_list_append (list, strdup (fullpath));
            }
        }
    }
    closedir (dev);
    list = g_list_sort (list, (GCompareFunc) compare);
    return list;
}

LibHalContext *
hal_init ()
{
    LibHalContext *ctx;
    ctx = libhal_ctx_new ();
    if (!ctx)
        return NULL;

    DBusConnection *bus = dbus_bus_get (DBUS_BUS_SYSTEM, NULL);
    if (!bus)
        return NULL;

    if (!libhal_ctx_set_dbus_connection (ctx, bus))
    {
        libhal_ctx_free (ctx);
        return NULL;
    }

    if (!libhal_ctx_init (ctx, NULL))
    {
        libhal_ctx_free (ctx);
        return NULL;
    }

    return ctx;
}

void
hal_shutdown (LibHalContext *ctx)
{
    libhal_ctx_shutdown (ctx, NULL);
    libhal_ctx_free (ctx);
}

char *
get_vendor_str (LibHalContext *ctx, const char *dev)
{
    char string[50]; /* we'll truncate if needed */
    LibHalDrive *dr = libhal_drive_from_device_file (ctx, dev);
    if (!dr)
        return NULL;

    const char *model = libhal_drive_get_model (dr);
    const char *vendor = libhal_drive_get_vendor (dr);

    if (model || vendor)
        if (snprintf (string, 50, "%s %s (%s)",
            vendor ? vendor : "", model ? model : "", dev) > 0)
                return g_strdup (string);
    return g_strdup (dev);
}
