/**
 * Copyright (C) 2009 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <glib.h>
#include <glib/gstdio.h>

#include "main.h"

GtkWidget *login_entry, *passwd_entry;
GtkWidget *fail_label;

gchar *old_login = NULL;

static void
get_old_login ()
{
    gchar  *cache_file;
    XfceRc *rc;

    cache_file = xfce_resource_save_location (
            XFCE_RESOURCE_CACHE, "linutop/updaterc", FALSE);
    if (! cache_file)
        return;

    rc = xfce_rc_simple_open (cache_file, TRUE);
    g_free (cache_file);
    if (!rc)
        return;

    old_login = g_strdup (xfce_rc_read_entry (rc, "login", ""));
    xfce_rc_close (rc);
}

static void
set_old_login (const gchar *login)
{
    gchar  *cache_file;
    XfceRc *rc;

    cache_file = xfce_resource_save_location (
            XFCE_RESOURCE_CACHE, "linutop/updaterc", TRUE);
    if (! cache_file)
        return;

    rc = xfce_rc_simple_open (cache_file, FALSE);
    g_free (cache_file);
    if (!rc)
        return;

    xfce_rc_write_entry (rc, "login", login);
    xfce_rc_close (rc);
}

static void
on_dialog_response (GtkDialog *dialog, gint arg1, gpointer data)
{
    if (arg1 == GTK_RESPONSE_OK) {
        status *s;
        gboolean anonymous = FALSE;
        const gchar *login;
        const gchar *passwd;
        const gchar *os;

        login = g_strdup (gtk_entry_get_text (GTK_ENTRY (login_entry)));
        passwd  = g_strdup (gtk_entry_get_text (GTK_ENTRY (passwd_entry)));
        os = get_os ();
        s = get_informations (login, passwd, os);

        if (!s)
            gtk_widget_show (fail_label);

        /* retry with anonymous login if we got wrong credentials */
        if (s->error && !strncmp ("Wrong login", s->error, 11))
        {
            s = get_informations (NULL, NULL, os);
            anonymous = TRUE;
        }

        if (!s)
        {
            gtk_widget_show (fail_label);
            _exit (0);
        } else {
            gtk_widget_destroy (GTK_WIDGET (dialog));
            if (login && strcmp (login, ""))
                set_old_login (login);

            if (anonymous)
            {
                update_dialog (s, NULL, NULL);
            } else {
                update_dialog (s, login, passwd);
            }
        }
    } else {
        gtk_main_quit ();
    }
}

void
login (gboolean mandatory)
{
    GtkWidget *dialog;
    GtkWidget *login_btn;
    GtkWidget *table, *label;

    fail_label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (fail_label),
                          "<span foreground=\"red\"><b>"
                          "Wrong login informations"
                          "</b></span>");

    dialog = xfce_titled_dialog_new_with_buttons (
            "Login",
            NULL,
            GTK_DIALOG_MODAL | GTK_DIALOG_NO_SEPARATOR,
            NULL, NULL);
    xfce_titled_dialog_set_subtitle (XFCE_TITLED_DIALOG (dialog),
                                     "Login to Linutop Update web services");
    /* TODO: add an icon to the header */

    login_btn = xfce_create_mixed_button (GTK_STOCK_OK, _("Login"));
    gtk_dialog_add_action_widget (GTK_DIALOG (dialog), login_btn,
                                  GTK_RESPONSE_OK);
    gtk_widget_show (login_btn);

    if (! mandatory)
    {
        label = gtk_label_new (
            "You can log in here if you have a dedicated Linutop Update account.\n"
            "Clic the 'Login' button for anonymous login");
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), label,
            FALSE, FALSE, 2);
        gtk_widget_show (label);
    }

    table = gtk_table_new (3, 2, FALSE);
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), table);
    gtk_widget_show (table);

    /* we hide the error message but it's there */
    gtk_table_attach (GTK_TABLE (table), fail_label, 0, 2, 0, 1,
                      GTK_EXPAND | GTK_FILL, GTK_FILL, 2, 2);

    label = gtk_label_new ("Login: ");
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
                      GTK_SHRINK, GTK_SHRINK, 2, 2);
    gtk_widget_show (label);

    label = gtk_label_new ("Password: ");
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
                      GTK_SHRINK, GTK_SHRINK, 2, 2);
    gtk_widget_show (label);

    login_entry = gtk_entry_new ();
    get_old_login ();
    if (old_login)
        gtk_entry_set_text (GTK_ENTRY (login_entry), old_login);
    gtk_table_attach (GTK_TABLE (table), login_entry, 1, 2, 1, 2,
                      GTK_EXPAND | GTK_FILL, GTK_SHRINK, 2, 2);
    gtk_widget_show (login_entry);

    passwd_entry = gtk_entry_new ();
    gtk_entry_set_visibility (GTK_ENTRY (passwd_entry), FALSE);
    gtk_table_attach (GTK_TABLE (table), passwd_entry, 1, 2, 2, 3,
                      GTK_EXPAND | GTK_FILL, GTK_SHRINK, 2, 2);
    if (old_login && strcmp (old_login, ""))
        gtk_widget_grab_focus (passwd_entry);
    gtk_widget_show (passwd_entry);

    g_signal_connect (G_OBJECT (dialog), "response",
                      G_CALLBACK (on_dialog_response), NULL);

    gtk_widget_show (dialog);
}
